using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;

namespace colorsquare
{
	public class Squares
	{
		private List<Texture2D> textures = new List<Texture2D> ();
		private List<Color> colors = new List<Color>
            {
                new Color (200, 200, 0), new Color (200, 0, 200),
				new Color (100, 240, 100), new Color (0, 200, 200), 
				new Color (240, 100, 100), new Color (100, 100, 240)
            };
		private List<Rectangle> rectangles = new List<Rectangle> ();
		private const int size = 22, spacing = 40;
		private Dictionary<Point, Boolean> hidden = new Dictionary<Point, Boolean> ();

		public Squares ()
		{
		}

		public void LoadContent (GraphicsDevice graphics_device)
		{
			foreach (Color color in colors) {
				textures.Add (new Texture2D (graphics_device, 1, 1));
				textures [textures.Count - 1].SetData<Color> (new Color[] { color });
			}
		}

		public void Update (Point dim, Vector2 offset, Rectangle avatar_bounds)
		{
			Rectangle test_rectangle;
			rectangles = new List<Rectangle> ();
			Point snapped_offset = new Point (grid_snap_floor(offset.X), grid_snap_floor(offset.Y));

			for (int i = 0; i < dim.X + spacing; i += spacing) {
				for (int j = 0; j < dim.Y + spacing; j += spacing) {
					test_rectangle = new Rectangle (
						i + snapped_offset.X,
						j + snapped_offset.Y,
						size,
						size);
					if (!hidden.ContainsKey(test_rectangle.Location)) {
						if (test_rectangle.Intersects (avatar_bounds)) {
							hidden [test_rectangle.Location] = true;
						} else
							rectangles.Add (test_rectangle);
					}
				}
			}
		}

		public void Draw (Vector2 offset, SpriteBatch spriteBatch)
		{
			foreach (var rectangle in rectangles) {
				spriteBatch.Draw (
					textures [get_color (rectangle.X, rectangle.Y)],
					new Rectangle (
                    rectangle.X - (int)offset.X,
					rectangle.Y - (int)offset.Y,
					rectangle.Width,
					rectangle.Height),
					Color.White);
			}
		}

		private int get_color (int x, int y)
		{
			return (int)(Math.Abs (x) / spacing + Math.Abs (y) / spacing) % 6;
		}

		private int grid_snap_floor (float a)
		{
			return (int)Math.Floor (a / spacing) * spacing;
		}
	}

}
